package org.telegram.ui.Business;

import android.util.SparseArray;

import org.telegram.messenger.AndroidUtilities;
import org.telegram.messenger.MessagesController;
import org.telegram.messenger.UserConfig;
import org.telegram.messenger.Utilities;
import org.telegram.tgnet.ConnectionsManager;
import org.telegram.tgnet.TLRPC;

import java.util.ArrayList;

public class BusinessChatbotController {

    private static volatile SparseArray<BusinessChatbotController> Instance = new SparseArray<>();
    private static final Object lockObject = new Object();

    public static BusinessChatbotController getInstance(int num) {
        BusinessChatbotController localInstance = Instance.get(num);
        if (localInstance == null) {
            synchronized (lockObject) {
                localInstance = Instance.get(num);
                if (localInstance == null) {
                    Instance.put(num, localInstance = new BusinessChatbotController(num));
                }
            }
        }
        return localInstance;
    }

    private final int currentAccount;
    private BusinessChatbotController(int account) {
        this.currentAccount = account;
    }

    private long lastTime;
    private TLRPC.TL_account_connectedBots value;
    private ArrayList<Utilities.Callback<TLRPC.TL_account_connectedBots>> callbacks = new ArrayList<>();
    private boolean loading, loaded;

    public void load(Utilities.Callback<TLRPC.TL_account_connectedBots> callback) {
        callbacks.add(callback);
        if (loading) return;
        if (System.currentTimeMillis() - lastTime > 1000 * 60 || !loaded) {
            loading = true;
            ConnectionsManager.getInstance(currentAccount).sendRequest(new TLRPC.TL_account_getConnectedBots(), (res, err) -> AndroidUtilities.runOnUIThread(() -> {
                loading = false;
                value = res instanceof TLRPC.TL_account_connectedBots ? (TLRPC.TL_account_connectedBots) res : null;
                if (value != null) {
                    MessagesController.getInstance(currentAccount).putUsers(value.users, false);
                }
                lastTime = System.currentTimeMillis();
                loaded = true;

                for (int i = 0; i < callbacks.size(); ++i) {
                    if (callbacks.get(i) != null) {
                        callbacks.get(i).run(value);
                    }
                }
                callbacks.clear();
            }));
        } else if (loaded) {
            for (int i = 0; i < callbacks.size(); ++i) {
                if (callbacks.get(i) != null) {
                    callbacks.get(i).run(value);
                }
            }
            callbacks.clear();
        }
    }

    public void invalidate() {
        loaded = false;
        load(null);
    }
}
